/// <reference types="isaac-typescript-definitions" />
import type { PlayerStat } from "../../../enums/PlayerStat";
import type { PlayerHealth } from "../../../interfaces/PlayerHealth";
import type { PlayerStats } from "../../../interfaces/PlayerStats";
import { Feature } from "../../private/Feature";
export declare class EdenStartingStatsHealth extends Feature {
    /**
     * We must use the `POST_PLAYER_INIT` callback since the two random collectibles have not been
     * granted yet.
     */
    private readonly postPlayerInit;
    private getEdenStats;
    private getEdenHealth;
    /**
     * Helper function to get the health that Eden started with at the beginning of the run before any
     * of the random collectibles were added.
     *
     * Returns undefined if passed a player that is not Eden.
     *
     * In order to use this function, you must upgrade your mod with `ISCFeature.EDEN_STARTING_STATS`.
     */
    getEdenStartingHealth(player: EntityPlayer): Readonly<PlayerHealth> | undefined;
    /**
     * Helper function to get the value of the randomized starting stat for Eden that was assigned at
     * the beginning of the run before any of the random collectibles were added.
     *
     * Returns undefined if passed a player that is not Eden.
     *
     * In order to use this function, you must upgrade your mod with `ISCFeature.EDEN_STARTING_STATS`.
     */
    getEdenStartingStat<T extends PlayerStat>(player: EntityPlayer, playerStat: T): PlayerStats[T] | undefined;
    /**
     * Helper function to get all of the stat values that Eden started with at the beginning of the
     * run before any of the random collectibles were added.
     *
     * Returns undefined if passed a player that is not Eden.
     *
     * In order to use this function, you must upgrade your mod with `ISCFeature.EDEN_STARTING_STATS`.
     */
    getEdenStartingStats(player: EntityPlayer): Readonly<PlayerStats> | undefined;
}
//# sourceMappingURL=EdenStartingStatsHealth.d.ts.map